Unit BasicTx;

{
 Version 1.02 av BasicTx - 991230 by HTH

 This minimal example shows how to send a S.N.A.P packet on the serialport.
 For this example to function properly you must install M. Cocco's COMDRV32
 component. For more information about this component see COMDRV32 documentation.

 Experienced programmers that want to take full advantage of the SNAP.DLL
 and implement functions like statistics, spying on other network traffic
 etc. see other examples in the ZIP-file.

 You can download the complete protocol description in PDF-format at
 the following URL...

   http://www.hth.com/snap/

 More information about PLM-24 Power Line Modem and related products
 is available at...

   http://www.hth.com/plm-24/

 Feel free to e-mail us any comments at info@hth.com. Due to the amount
 of e-mails we receive we are not always able to reply to you personally
 but all comments about the DLL or S.N.A.P network protocol in general
 are read and all feedback is appreciated!

 Disclaimer
 ----------

 This is released as "AS IS" and we are not able to provide any free
 support.

 Copyright (c) 1999 HTH all rights reserved

 Delphi 3

 External Components/DLL files

 Name         Version       Maker
 ------------------------------------
 COMDRV32      1.08        M. Cocco
 SNAP.DLL      1.10        Harry Zachrisson, HTH
  }
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComDrv32;


CONST
  SYNCSIZE=1;
  {Define maxsize in bytes of different portions of the protocoll}
  MAXADDRESSSIZE=3;
  MAXPFBFLAGS=3;
  MAXUFBFLAGS=3;
  MAXDATASIZE=512;
  MAXEDMSIZE=4;
  MAXPACKSIZE=1042;
  SYNC=84;
  ACK:Byte=1;
  NAK:Byte=0;
  WANTACK:Byte=1;
  DONTWANTACK:Byte=0;
  PACKBUFFSIZE=50;
  RX=1;
  TX=2;
  SPY=3;
  NOERR=0;
  RETRAN8=1;
  SUM8=2;
  CRC8=3;
  CRC16=4;
  CRC32=5;
  FEC=6;

Type
  Type_Version= Record
                  Major  :Byte;
                  Minor  :Byte;
                End;

  Type_Statistics=Record
                    RejectedPackets:LongInt;
                    TransmittedPackets:LongInt;
                    ReceivedPackets:LongInt;
                    TotalPackets:LongInt;
                    RejectedBytes:LongInt;
                    TransmittedBytes:LongInt;
                    ReceivedBytes:LongInt;
                    TotalBytes:LongInt;
                    StartTime:TDateTime;
                  End;

  Type_NumOfBytes=Array [0..3] of Word;
  Type_NumOfErrorBytes=Array [0..7] of Word;
  Type_NumOfDataBytes=Array [0..15] of Word;
  Type_HeaderDef = Record
                     DAB,SAB,PFB,ACK,RESERVED,EDM,NDB :Byte;
                   End;

  Type_ByteBuff=Array [1..MAXPACKSIZE] of Byte;

  Type_DataBytes= Array [1..MAXDATASIZE]of Byte; {Un/decoded data from or to DLL from App.}
  Type_PacketBytes= Array [1..MAXPACKSIZE]of Byte; {SNAP Packetbytes from or to Comlayer }

  Type_Packet = Record
                  NumOfBytes:SmallInt;           {Number of bytes in packet}
                  PacketBytes:Type_PacketBytes;  {All bytes in packet execept leading preamble bytes}
                  Time:TDateTime;                {Time stamp}
                End;

  Type_RXData = Record
                  DestAdd:LongInt;         {-1=No Destination address was used. 0=Broadcast}
                  SourceAdd:LongInt;       {-1=No Source address was used. 0=Broadcast}
                  Data:Type_DataBytes;     {All databytes}
                  NumOfDataBytes:SmallInt; {0-MAXDATASIZE}
                  EDMBits:Byte;            {only bits 0-3 is used}
                  ACKBits:Byte;            {only bits 0-3 is used}
                  CMDBit:Byte;             {only bit 0 is used}
                  RXTime:TDateTime;        {Time stamp}
                End;

   Type_TXData = Record
                   DestAdd:LongInt;         {-1=No Destination address will be used. 0=Broadcast}
                   Data:Type_DataBytes;     {All databytes}
                   NumOfDataBytes:SmallInt; {0-MAXDATASIZE}
                   ErrorMode:Byte;          {Errordetection mode}
                   Ack:Byte;                {ACK Bits}
                   Cmd:Byte;                {Sets command mode bit}
                 End;


  TFormBasicTx = class(TForm)
    ComPortDriver: TCommPortDriver;
    ButtonSend: TButton;
    Labelconnect: TLabel;
    TXTimer: TTimer;
    procedure FormActivate(Sender: TObject);

    procedure ButtonSendClick(Sender: TObject);
    procedure TXTimerTimer(Sender: TObject);
  private
    {Private declarations}
  public
    {Public declarations}
  end;

var
  FormBasicTx: TFormBasicTx;

implementation

{$R *.DFM}

{Turn on or off printing of debug info to a textfile}
Procedure SetDebugMode (Debug:WordBool) stdcall ; external 'snap.dll' name 'SetDebugMode';
{Return DLL version to Application}
Function DLLVersion:Type_Version;  stdcall;external 'snap.dll' name 'DLLVersion';
{Sets the address on this node}
Procedure SetMyAddress (MyAddress:LongInt) stdcall ; external 'snap.dll' name 'SetMyAddress';
{Return my node address}
Function MyAddress :LongInt stdcall ;external 'snap.dll' name 'MyAddress';
{Return statistics counters}
Procedure GetStatistics (VAR Stat:Type_Statistics) stdcall;external 'snap.dll' name 'MyAddress';
{Send Data (put in TX buffert)}
Function SendData(VAR TXData:Type_TXData) :WordBool stdcall ;external 'snap.dll' name 'SendData';
{Read back a TX packet }
Function GetTXPacket (VAR OutPacket:Type_Packet):WordBool stdcall; external 'snap.dll' name 'GetTXPacket';
{Input Bytes to ByteBuffert}
Function RXByteIn (InByte:byte) :WordBool stdcall ;external 'snap.dll' name 'RXByteIn';
{Read a RX packet }
Function GetRXData (VAR RXData:Type_RXData):WordBool stdcall; external 'snap.dll' name 'GetRXData';
{Read a SPY packet }
Function GetSPYData (VAR RXData:Type_RXData):WordBool stdcall; external 'snap.dll' name 'GetSpyData';


procedure TFormBasicTx.FormActivate(Sender: TObject);
begin
{Initialise com port settings}
ComPortDriver.ComPort := pnCOM1;
ComPortDriver.ComPortDataBits := db8BITS;
ComPortDriver.ComPortSwHandshaking := shNONE;
ComPortDriver.ComPortHwHandshaking := hhNONE;
ComPortDriver.ComPortParity := ptNONE;
ComPortDriver.ComPortSpeed := br2400;
ComPortDriver.ComPortStopBits := sb1BITS;
ComPortDriver.EnableDTROnOpen := false;
if ComportDriver.connect then begin
  Labelconnect.visible := true
end;
end;{TFormBasicTx.FormActivate}

procedure TFormBasicTx.TXTimerTimer(Sender: TObject);
Var
  ByteCount:Integer;
  Packet:Type_Packet;
  Succes:Boolean;
begin
While GetTXPacket (Packet) Do {If a packet is ready to be sent}
  Begin
  TXTimer.Enabled:=False; {Disable TXTimer while sending}
  For ByteCount:=1 to Packet.NumOfBytes Do {Send byte by byte}
    Begin
    Succes:=ComPortDriver.SendByte(Packet.PacketBytes[ByteCount]);
    If NOT Succes then Begin {Unable to send byte to serial port}
      Beep;
      exit;
    End;{IF NOT Succes}
  End; {For ByteCount}
End; {While}
TXTimer.Enabled:=True; {Re-enable TXTimer}
end; {TFormBasicTx.TXTimerTimer}


procedure TFormBasicTx.ButtonSendClick(Sender: TObject);
VAR
  TX:Type_TXData;
begin
  TX.Data[1]:=255;      {Fill first databyte (DB1) with value 255}
  TX.DestAdd:=3;        {Send to node number 3}
  TX.NumOfDataBytes:=1; {Send one data byte}
  TX.ErrorMode:=CRC16;  {Use CRC-16 as error detection mode}
  TX.Ack:=WANTACK;      {Request ACK}
  TX.Cmd:=0;            {Set command mode bit=0}
  SendData (TX);        {Send data to the DLL and place the packet in the outbuffer}
end;


begin
  SetMyAddress (1);  {We are number one :o)}

  {Since this program is event and timer driven this is all the
  code needed in the main procedure}
  
end.
